<?php
/**
 * @package			No Boss Extensions
 * @subpackage  	No Boss Banners
 * @author			No Boss Technology <contact@nobosstechnology.com>
 * @copyright		Copyright (C) 2025 No Boss Technology. All rights reserved.
 * @license			GNU Lesser General Public License version 3 or later; see <https://www.gnu.org/licenses/lgpl-3.0.en.html>
 */

namespace Noboss\Module\Nobossbanners\Site\Dispatcher;

use Joomla\CMS\Dispatcher\AbstractModuleDispatcher;
use Joomla\CMS\Helper\HelperFactoryAwareInterface;
use Joomla\CMS\Helper\HelperFactoryAwareTrait;
use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;
use Noboss\Library\Util\NbUrlUtil;
use Noboss\Library\Util\NbFontsUtil;
use Noboss\Library\Util\NbLoadextensionAssetsUtil;

// phpcs:disable PSR1.Files.SideEffects
\defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * Dispatcher class
 */
class Dispatcher extends AbstractModuleDispatcher implements HelperFactoryAwareInterface {
    use HelperFactoryAwareTrait;

    /**
     * Returns the layout data.
     *
     * @return  array
     */
    protected function getLayoutData(){
        $data = parent::getLayoutData();

        // Para testes de exeucao direto do script de update
        // require_once JPATH_SITE.'/modules/mod_nobossbanners/script.extension.php';
        // \mod_NobossbannersInstallerScript::updateNewVersion();
        // exit;

        // Classe helper
        $helperClass = $this->getHelperFactory()->getHelper('NobossbannersHelper');

        // Verifica se no boss library esta instalada: se ainda nao estiver, tenta instalar e se ainda nao der retorna false
        if(!$helperClass->checkLibraryInstallation()){
            return false;
        }

        // Alias da extensao (ex: nobossbanners)
        $data['module']->name = str_replace('mod_', '', $data['module']->module);

        // Carrega arquivo traducao da library
        $assetsObject = new NbLoadextensionAssetsUtil('lib_noboss');
        $extensionPath = $assetsObject->getDirectoryExtension(false);
        Factory::getLanguage()->load("lib_noboss", $extensionPath);

        // Model do layout escolhido
        $theme = json_decode($data['params']->get("theme"))->theme;

        switch ($theme) {
            case 'model1':
                $data['layoutAlias'] = 'fullpage';
                break;
            case 'model2':
                $data['layoutAlias'] = 'parallax';
                break;
            case 'model3':
                $data['layoutAlias'] = 'carrossel';
                break;
        }

        // Obtem banners cadastrados
        $banners = (array) $data['params']->get($theme);

        // Layout carrossel
        if($theme == "model3"){
            // Objeto com os parametros de customizacao do layout dos items
            $layoutCustom = json_decode($data['params']->get("{$theme}_items_customization"));
            
            // Parametros para o plugin de slider 
            $jsConfig = new \stdClass();
            $jsConfig->autoPlay = $layoutCustom->enable_auto_play ? true : false;
            $jsConfig->interval = $layoutCustom->enable_auto_play ? $layoutCustom->auto_play_interval : 5;
            $jsConfig->dragging = $layoutCustom->enable_dragging ? true : false;
            $jsConfig->showDots = $layoutCustom->show_dots ? true : false;

            // Modo de exibicao (1- manual / 0- full width)
            if(isset($layoutCustom->content_display_mode) && $layoutCustom->content_display_mode){
                $itemColumns = !empty($layoutCustom->content_columns) ? "nb-lg-{$layoutCustom->content_columns} nb-md-{$layoutCustom->content_columns} nb-sm-{$layoutCustom->content_columns} nb-xs-{$layoutCustom->content_columns}" : "";
            }
            
            // Setas
            $arrowsStyle = "font-size: inherit; ";
            $arrowsStyleHover = "";
            $layoutCustom->arrows_color = !empty($layoutCustom->arrows_color) ? $layoutCustom->arrows_color : '';
            $showArrows = $layoutCustom->show_arrows != 'none';
            if($showArrows){
                $arrowsIcon = $layoutCustom->arrows_icon;

                // Icone do 'font-awesome-v6'
                if(strpos($arrowsIcon, 'fa-') !== false){
                    $arrowFamilyIcons = "font-awesome-v6";
                    $layoutCustom->arrows_class_left = "fa ".str_replace('DIRECTION', 'left', $arrowsIcon);
                    $layoutCustom->arrows_class_right = "fa ".str_replace('DIRECTION', 'right', $arrowsIcon);
                }
                // Icone do 'simple-line'
                else if(strpos($arrowsIcon, 'icon-') !== false){
                    $arrowFamilyIcons = "simple-line";
                    $layoutCustom->arrows_class_left = str_replace('DIRECTION', 'left', $arrowsIcon);
                    $layoutCustom->arrows_class_right = str_replace('DIRECTION', 'right', $arrowsIcon);
                }
                // Icone do 'linear-icons'
                else if(strpos($arrowsIcon, 'lnr') !== false){
                    $arrowFamilyIcons = "linear-icons";
                    $layoutCustom->arrows_class_left = str_replace('DIRECTION', 'left', $arrowsIcon);
                    $layoutCustom->arrows_class_right = str_replace('DIRECTION', 'right', $arrowsIcon);
                }
                // Icone do 'material-design'
                else if(strpos($arrowsIcon, '_') !== false){
                    $arrowFamilyIcons = "material-design";
                    $layoutCustom->arrows_class_left = "material-symbols-outlined";
                    $layoutCustom->arrows_class_right = "material-symbols-outlined";
                    $layoutCustom->arrows_content_left = str_replace('DIRECTION', 'left', $arrowsIcon);
                    $layoutCustom->arrows_content_right = str_replace('DIRECTION', 'right', $arrowsIcon);
                }

                $arrowsIconSize = $layoutCustom->arrows_icon_size;
                if(!empty($arrowsIconSize)){
                    $arrowsIconSizeEm = $arrowsIconSize/16;
                    $arrowsIconSize = "font-size: {$arrowsIconSize}px; font-size: {$arrowsIconSizeEm}em;";
                }
                $arrowsStyle .= ' color:' . $layoutCustom->arrows_color . ';';
                $arrowsStyleMobile = "";
                if(isset($layoutCustom->arrows_icon_size_mobile)){
                    $arrowIconSizeMobile = $layoutCustom->arrows_icon_size_mobile;
                    $arrowIconSizeMobileEm = $arrowIconSizeMobile/16;
                    $arrowsStyleMobile =  "font-size: {$arrowIconSizeMobile}px; font-size: {$arrowIconSizeMobileEm}em !important;";
                }
            }

            // Dots
            $showDots = $jsConfig->showDots;
            $dotsStyle = "";
            $dotsStyle .= "background-color: rgba(255, 255, 255, 0); ";
            $dotsStyle .= "margin: 0 " . $layoutCustom->dots_spacing_items . "px; ";
            $dotsStyle .= "border: 2px solid " . $layoutCustom->arrows_color . "; ";
            $dotsStyle .= "border-radius: {$layoutCustom->dots_border_radius_size}px; ";           
            $dotsStyle .= "padding: " . implode(' ', $layoutCustom->dots_size) . "; ";
            $dotsStyleMobile = "";
            $layoutCustom->dots_size_mobile = !empty($layoutCustom->dots_size_mobile) ? $layoutCustom->dots_size_mobile : array('5px', '5px');
            $dotsStyleMobile .= "padding: " . implode(' ', $layoutCustom->dots_size_mobile) . "; ";
            $dotsActiveStyle = "";
            $dotsActiveStyle .= "background-color:  " . $layoutCustom->arrows_color . "; ";

            // Altura do banner
            $height = $layoutCustom->height . 'px';
            $heightMobile = 'height: ' . $layoutCustom->height_mobile . 'px !important;';

            // Obtem array com niveis de acesso do usuario (se nao estiver autenticado vem apenas '1' de public)
            $userAccessLevels = Factory::getApplication()->getIdentity()->getAuthorisedViewLevels();
        }

        $nowDate = date('Y-m-d H:i:s');

        // Verifica se foi cadastrado ao menos um
        if (count($banners) > 0){
            
            // Percorre todos os banners
            foreach ($banners as $key => &$bannerItem) {              
                // Layout carrossel
                if($theme == "model3"){
                    // Se nivel de acesso nao estiver definido, coloca 'public' como default
                    $bannerItem->access = isset($bannerItem->access) ? $bannerItem->access : '1';

                    // Item do Banner nao esta publicado
                    if ($bannerItem->published != 1){
                        unset($banners[$key]);
                        continue;
                    }
                    // Usuario nao possui permissao para visualizar o item do banner
                    else if (!in_array($bannerItem->access, $userAccessLevels)) {
                        unset($banners[$key]);
                        continue;
                    }
                    // Data de publicacao de inicio esta definida e eh maior do que a data atual
                    else if ((($bannerItem->publish_in != '') && ($bannerItem->publish_in != '0000-00-00 00:00:00')) &&
                    (strtotime($bannerItem->publish_in) >= strtotime($nowDate))){
                        unset($banners[$key]);
                        continue;
                    }
                    // Data de publicacao de fim esta definida e eh menor do que a data atual
                    else if ((($bannerItem->publish_down != '') && ($bannerItem->publish_down != '0000-00-00 00:00:00')) &&
                    (strtotime($nowDate) >= strtotime($bannerItem->publish_down))){
                        unset($banners[$key]);
                        continue;
                    }
                }

                // Modal de personalizacao de imagem overlay
                $bannerItem->overlay = json_decode($bannerItem->overlay);
                $bannerItem->overlay = (!empty($bannerItem->overlay)) ? $bannerItem->overlay : new \stdClass();

                // Modal de personalizacao posicionalmento e estilo dos textos
                $bannerItem->texts_custom = json_decode($bannerItem->texts_custom);
                $bannerItem->texts_custom = (!empty($bannerItem->texts_custom)) ? $bannerItem->texts_custom : new \stdClass();

                // Se imagem estiver definida e nao for url completa, coloca url base do site e tira '#' com conteudo extra que Joomla coloca
                $bannerItem->image = (!empty($bannerItem->image) && (!filter_var($bannerItem->image, FILTER_VALIDATE_URL))) ? URI::root().explode("#", $bannerItem->image)[0] : $bannerItem->image;
                $bannerItem->image_mobile = (!empty($bannerItem->image_mobile) && (!filter_var($bannerItem->image_mobile, FILTER_VALIDATE_URL))) ? URI::root().explode("#", $bannerItem->image_mobile)[0] : $bannerItem->image_mobile;
                $bannerItem->overlay->image = (!empty($bannerItem->overlay->image) && (!filter_var($bannerItem->overlay->image, FILTER_VALIDATE_URL))) ? URI::root().explode("#", $bannerItem->overlay->image)[0] : $bannerItem->overlay->image;

                // Posicao dos textos no desktop
                $bannerItem->texts_custom->text_position = !empty($bannerItem->texts_custom->text_position) ? $bannerItem->texts_custom->text_position : 'center_center';
                $temp = explode("_", $bannerItem->texts_custom->text_position);
                $bannerItem->texts_custom->content_position = "nb-vertical--".$temp[0];
                $bannerItem->texts_custom->content_position .= " nb-horizontal--".$temp[1];

                // Posicao dos textos no mobile
                $bannerItem->texts_custom->text_position_mobile = !empty($bannerItem->texts_custom->text_position_mobile) ? $bannerItem->texts_custom->text_position_mobile : 'center_center';
                $temp = explode("_", $bannerItem->texts_custom->text_position_mobile);
                $bannerItem->texts_custom->content_position .= " nb-vertical-mobile--".$temp[0];
                $bannerItem->texts_custom->content_position .= " nb-horizontal-mobile--".$temp[1];  
                
                // Area de conteudo
                $bannerItem->texts_custom->content_alignment = (!empty($bannerItem->texts_custom->content_alignment)) ? $bannerItem->texts_custom->content_alignment : 'center';
                $bannerItem->texts_custom->content_color = (!empty($bannerItem->texts_custom->content_color)) ? $bannerItem->texts_custom->content_color : '#616161';
                'center';
                $bannerItem->texts_custom->content_color_mobile = (!empty($bannerItem->texts_custom->content_color_mobile)) ? $bannerItem->texts_custom->content_color_mobile : $bannerItem->texts_custom->content_color;
                $bannerItem->texts_custom->line_spacing = (!empty($bannerItem->texts_custom->line_spacing)) ? $bannerItem->texts_custom->line_spacing : '25';
                $bannerItem->texts_custom->line_spacing_mobile = (!empty($bannerItem->texts_custom->line_spacing_mobile)) ? $bannerItem->texts_custom->line_spacing_mobile : '15';
                $bannerItem->texts_custom->maxwidth = (!empty($bannerItem->texts_custom->maxwidth)) ? $bannerItem->texts_custom->maxwidth : '50';
                $bannerItem->texts_custom->maxwidth_mobile = (!empty($bannerItem->texts_custom->maxwidth_mobile)) ? $bannerItem->texts_custom->maxwidth_mobile : '90';
                $bannerItem->texts_custom->content_space = (!empty($bannerItem->texts_custom->content_space)) ? implode(' ', $bannerItem->texts_custom->content_space) : '20px 20px';
                $bannerItem->texts_custom->content_space_mobile = (!empty($bannerItem->texts_custom->content_space_mobile)) ? implode(' ', $bannerItem->texts_custom->content_space_mobile) : '15px 15px';               
                $bannerItem->texts_custom->content_margin = (!empty($bannerItem->texts_custom->content_margin)) ? implode(' ', $bannerItem->texts_custom->content_margin) : '0% 0% 0% 0%';
                $bannerItem->texts_custom->content_margin_mobile = (!empty($bannerItem->texts_custom->content_margin_mobile)) ? implode(' ', $bannerItem->texts_custom->content_margin_mobile) : '0% 0% 0% 0%';
                
                // Texto linha 1
                $bannerItem->title_style = "";
                $bannerItem->texts_custom->title_font = !empty($bannerItem->texts_custom->title_font) ? $bannerItem->texts_custom->title_font : '{"fontfamily":"Montserrat_Regular.ttf","externalLinked":"","fontStyle":"Bold"}';
                $bannerItem->title_style .= NbFontsUtil::importNobossfontlist($bannerItem->texts_custom->title_font);
                $bannerItem->title_style .= " text-align: {$bannerItem->texts_custom->content_alignment};";
                $itemTitleSize = !empty($bannerItem->texts_custom->title_size) ? $bannerItem->texts_custom->title_size : "";
                $itemTitleSizeMobile = !empty($bannerItem->texts_custom->title_size_mobile) ? $bannerItem->texts_custom->title_size_mobile : "";
                $bannerItem->title_style_mobile = "";
                if(!empty($itemTitleSize)){
                    $itemTitleSizeEm = $itemTitleSize/16;
                    $bannerItem->title_style .= " font-size: {$itemTitleSize}px; font-size: {$itemTitleSizeEm}em;";
                }
                if(!empty($itemTitleSizeMobile)){
                    $itemTitleSizeMobileEm = $itemTitleSizeMobile/16;
                    $bannerItem->title_style_mobile .= " font-size: {$itemTitleSizeMobile}px; font-size: {$itemTitleSizeMobileEm}em !important;";
                }
                
                // Texto linha 2
                $bannerItem->subtitle_style = "";
                $bannerItem->texts_custom->subtitle_style = !empty($bannerItem->texts_custom->subtitle_style) ? $bannerItem->texts_custom->subtitle_style : '{"fontfamily":"Merriweather_Regular.ttf","externalLinked":"","fontStyle":"Bold"}';
                $bannerItem->subtitle_style .= NbFontsUtil::importNobossfontlist($bannerItem->texts_custom->subtitle_style);
                $bannerItem->subtitle_style .= " text-align: {$bannerItem->texts_custom->content_alignment};";
                $itemSubtitleSize = $bannerItem->texts_custom->subtitle_size;
                $itemSubtitleSizeMobile = $bannerItem->texts_custom->subtitle_size_mobile;
                $bannerItem->subtitle_style_mobile = "";
                if(!empty($itemSubtitleSize)){
                    $itemSubtitleSizeEm = $itemSubtitleSize/16;
                    $bannerItem->subtitle_style .= " font-size: {$itemSubtitleSize}px; font-size: {$itemSubtitleSizeEm}em;";
                }
                if(!empty($itemSubtitleSizeMobile)){
                    $itemSubtitleSizeMobileEm = $itemSubtitleSizeMobile/16;
                    $bannerItem->subtitle_style_mobile .= " font-size: {$itemSubtitleSizeMobile}px; font-size: {$itemSubtitleSizeMobileEm}em !important;";
                }

                $bannerItem->background_style = "";

                // Full Page
                if($theme == "model1"){
                    // Possui imagem
                    if($bannerItem->show_image && $bannerItem->image != ''){
                        $bannerItem->background_style = "height: 100vh; background-size: cover; position: relative; background-position: center; background-image: url('" . $bannerItem->image . "');";
                    }
                    else{
                        $bannerItem->background_color = !empty($bannerItem->background_color) ? $bannerItem->background_color : '';
                        $bannerItem->background_style = "height: 100vh; position: relative; background-color: {$bannerItem->background_color};>";
                    }
                }

                // Full Page e Parallax
                if($theme != "model3"){
                    // Modal de largura do banner
                    $bannerItem->banner_width = json_decode($bannerItem->banner_width);
                    $bannerItem->banner_width = (!empty($bannerItem->banner_width)) ? $bannerItem->banner_width : new \stdClass();

                    // Definida altura no mobile
                    if(!empty($bannerItem->banner_width->height_mobile)){
                        $bannerItem->height_mobile = 'height: ' . $bannerItem->banner_width->height_mobile . 'px !important;';
                    }                   

                    // Configuracao de largura do banner
                    if(isset($bannerItem->banner_width->content_display_mode) && $bannerItem->banner_width->content_display_mode){
                        $itemColumns = !empty($bannerItem->banner_width->content_columns) ? "nb-lg-{$bannerItem->banner_width->content_columns} nb-md-{$bannerItem->banner_width->content_columns} nb-sm-{$bannerItem->banner_width->content_columns} nb-xs-{$bannerItem->banner_width->content_columns}" : "";
                    }
                }
                // Model 3
                else{
                    if($bannerItem->show_image && $bannerItem->image != ''){
                        $bannerItem->background_style = "height: {$height}; background-size: cover; position: relative; background-position: center; background-image: url('" . $bannerItem->image . "');";
                    }else{
                        $bannerItem->background_color = !empty($bannerItem->background_color) ? $bannerItem->background_color : '';
                        $bannerItem->background_style = "height: {$height}; position: relative; background-color: {$bannerItem->background_color};>";
                    }
                }

                // Parallax
                if($theme == "model2"){
                    $bannerItem->height = 'height: ' . $bannerItem->banner_width->height . 'px;';
                }
                
                // Possui imagem para o mobile
                if(!empty($bannerItem->image_mobile)){
                    $bannerItem->image_mobile = "background-image: url('" . $bannerItem->image_mobile . "') !important;";
                }
                $bannerItem->overlay->position = "";
                
                // Monta o estilo da altura da imagem overlay 
                if(!empty($bannerItem->overlay->image)){
                    $bannerItem->overlay->image_height = "height: " . $bannerItem->overlay->image_height . "px;";
                    $bannerItem->overlay->image_height_mobile = "height: " . $bannerItem->overlay->image_height_mobile . "px;";
                    $bannerItem->overlay->position = "nb-horizontal--{$bannerItem->overlay->image_horizontal_position} nb-vertical--{$bannerItem->overlay->image_vertical_position} nb-horizontal-mobile--{$bannerItem->overlay->image_horizontal_position_mobile} nb-vertical-mobile--{$bannerItem->overlay->image_vertical_position_mobile}";
                }
                
                $bannerItem->links = new \stdClass();
                if($bannerItem->show_link){
                    $bannerItem->links->link1 = json_decode($bannerItem->link1_customization);
                }
                
                if($bannerItem->show_link == 'button' && $bannerItem->show_link2){
                    $bannerItem->links->link2 = json_decode($bannerItem->link2_customization);
                }
                $count = 0;
                $bannerItem->links_class = "";

                // Percorre os links do banner
                foreach ($bannerItem->links as $key => $link) {
                    if(empty($link)){
                        continue;
                    }

                    $count++;
                    
                    if($link->link_option == "external") {
                        $link->link_url = $link->link_external;
                    }
                    else if($link->link_option == "internal") {
                        $link->link_url = NbUrlUtil::getUrlItemMenu($link->link_internal);
                    }

                    $link->link_target = $link->link_target ? "_blank" : "_self";

                    if($bannerItem->show_link == 'button'){
                        $link->button_style = "";
                        $link->button_style_hover = "";
                        $link->buttons_text_color = !empty($link->buttons_text_color) ? $link->buttons_text_color : '';
                        $link->buttons_background_color = !empty($link->buttons_background_color) ? $link->buttons_background_color : '';
                        $link->buttons_hover_text_color = !empty($link->buttons_hover_text_color) ? $link->buttons_hover_text_color : '';
                        $link->buttons_background_hover_color = !empty($link->buttons_background_hover_color) ? $link->buttons_background_hover_color : '';

                        switch ($link->buttons_style) {
                            // Botao quadrado
                            case 'squared-button':
                                $link->button_style = "color: {$link->buttons_text_color}; background-color: {$link->buttons_background_color};";
                                $link->button_style_hover = "color: {$link->buttons_hover_text_color} !important; background-color: {$link->buttons_background_hover_color} !important;";
                                break;
                            // Botao arredondado
                            case 'rounded-button':
                                $link->button_style = "color: {$link->buttons_text_color}; background-color: {$link->buttons_background_color}; border-radius: {$link->buttons_border_radius_size}px;";
                                $link->button_style_hover = "color: {$link->buttons_hover_text_color} !important; background-color: {$link->buttons_background_hover_color} !important;";
                                break;
                            // Botao transparente quadrado
                            case 'ghost-squared-button':
                                $link->button_style = "background-color: transparent; border: 2px solid; color: {$link->buttons_text_color}; border-color: {$link->buttons_text_color};";
                                $link->button_style_hover = "color: {$link->buttons_hover_text_color} !important; border-color: {$link->buttons_hover_text_color} !important; background-color: {$link->buttons_background_hover_color} !important;";
                                break;
                            // Botao transparente arredondado
                            case 'ghost-rounded-button':
                                $link->button_style = "background-color: transparent; border: 2px solid; color: {$link->buttons_text_color}; border-color: {$link->buttons_text_color}; border-radius: {$link->buttons_border_radius_size}px;";
                                $link->button_style_hover = "color: {$link->buttons_hover_text_color} !important; border-color: {$link->buttons_hover_text_color} !important; background-color: {$link->buttons_background_hover_color} !important;";
                                break;

                            // Botao com borda somente abaixo
                            case 'border-below-button':
                                $link->button_style = "background-color: transparent; border: 2px solid; border-width: 0 0 1px 0; color: {$link->buttons_text_color}; border-color: {$link->buttons_text_color};";
                                $link->button_style_hover = "color: {$link->buttons_hover_text_color} !important; border-color: {$link->buttons_hover_text_color} !important;";
                                break;

                            // Botao link simples
                            case 'simple-link':
                                $link->button_style = "color: {$link->buttons_text_color};";
                                $link->button_style_hover = "color: {$link->buttons_hover_text_color} !important";
                                break;

                            default:
                                $link->button_style = "color: {$link->buttons_text_color};";
                                break;
                        }

                        $link->button_style .= NbFontsUtil::importNobossfontlist($link->buttons_font);
                        
                        $bannerItem->links_class .= "button{$count}-{$bannerItem->texts_custom->content_alignment} ";
                        // $link->button_style .= " text-transform: " . $link->buttons_text_transform . ";";
                        $buttonsSize = $link->buttons_text_size;
                        if(!empty($buttonsSize)){
                            $buttonsSizeEm = $buttonsSize/16;
                            $link->button_style .= " font-size: {$buttonsSize}px; font-size: {$buttonsSizeEm}em;";
                        }

                        $link->button_style .= isset($link->buttons_space) ? ' padding: '.implode(' ', $link->buttons_space).';' : ' padding: 12px 20px;';
                    }
                }
            }

            // Armazena a quantidade de banners ativos 
            $availableItems = count($banners);

            // Possui banners a exibir
            if($availableItems > 0){
                if($theme == "model3"){
                    // manda o valor de banners ativos para o objeto js 
                    $jsConfig->bannersQtd = $availableItems;
                }
            
                // Define prefixo a ser utilizado na insercao de codigos inline para CSS e JS
                $prefixCodeJsAndCss = "[module-id=".$data['module']->name."_".$data['module']->id."]";
            
                // Obtem informacoes de codigos JS e CSS a serem inseridos
                $loadJs	 		= "1";
                $loadCss 		= "1";
                $overwritingJs	= (!empty($data['params']->get('overwriting_js'))) ? rtrim($data['params']->get('overwriting_js')) : '';
                $overwritingCss	= (!empty($data['params']->get('overwriting_css'))) ? rtrim($data['params']->get('overwriting_css')) : '';
            
                // Instancia objeto passando o nome da extensao com prefixo (ex: 'mod_nobossbanners')
                $assetsObject = new NbLoadextensionAssetsUtil($data['module']->module, $prefixCodeJsAndCss);
                
                // Adiciona arquivos e codigos JS (se definido para exibir)
                $assetsObject->loadJs($loadJs, array('code' => $overwritingJs));
            
                // Adiciona arquivo CSS para uma familia de fontes
                if(isset($arrowFamilyIcons)){
                    $assetsObject->loadFamilyIcons($arrowFamilyIcons);
                }

                // Adiciona arquivos e codigos CSS (se definido para carregar)
                $assetsObject->loadCss($loadCss, array('code' => $overwritingCss));
            
                // Define a tmpl a ser carregada
                $data['params']->set('layout', 'theme/'.$data['layoutAlias']);            
            }

            // Declaracao de variaveis para tmpls
            $data['banners'] = $banners;
            $data['jsConfig'] = isset($jsConfig) ? $jsConfig : '';
            $data['itemColumns'] = isset($itemColumns) ? $itemColumns : '';
            $data['layoutCustom'] = isset($layoutCustom) ? $layoutCustom : '';
            $data['showDots'] = isset($showDots) ? $showDots : '';
            $data['dotsStyle'] = isset($dotsStyle) ? $dotsStyle : '';
            $data['dotsStyleMobile'] = isset($dotsStyleMobile) ? $dotsStyleMobile : '';
            $data['dotsActiveStyle'] = isset($dotsActiveStyle) ? $dotsActiveStyle : '';
            $data['arrowsStyle'] = isset($arrowsStyle) ? $arrowsStyle : '';
            $data['arrowsStyleHover'] = isset($arrowsStyleHover) ? $arrowsStyleHover : '';
            $data['showArrows'] = isset($showArrows) ? $showArrows : '';
            $data['arrowsIcon'] = isset($arrowsIcon) ? $arrowsIcon : '';
            $data['arrowsIconSize'] = isset($arrowsIconSize) ? $arrowsIconSize : '';
            $data['arrowsStyleMobile'] = isset($arrowsStyleMobile) ? $arrowsStyleMobile : '';
            $data['height'] = isset($height) ? $height : '';
            $data['heightMobile'] = isset($heightMobile) ? $heightMobile : '';
            $data['theme'] = $theme;
            $data['prefixCodeJsAndCss'] = isset($prefixCodeJsAndCss) ? $prefixCodeJsAndCss : '';
            $data['assetsObject'] = $assetsObject;
            $data['availableItems'] = $availableItems;
        }

        return $data;
    }
}
